#include <cstdlib>
#include <iostream>
#include <algorithm>
#include "m3cLogger.h"

#ifdef MELISSA_ANDROID
    #include <android/log.h>
#endif

namespace melissa
{

m3cLoggerStream::m3cLoggerStream(const m3cLoggerStream &other)
    : m_fatal(other.m_fatal),
      m_prefix(other.m_prefix)
{
    other.m_fatal = false;
}

m3cLoggerStream::m3cLoggerStream(const std::string &prefix, bool fatal)
    : m_fatal(fatal),
      m_prefix(prefix)
{
    out << m_prefix;
}

m3cLoggerStream::~m3cLoggerStream()
{
    std::cout << out.str() << std::endl;
#ifdef MELISSA_ANDROID
    __android_log_print(m_fatal ? ANDROID_LOG_ERROR : ANDROID_LOG_WARN, "MELISSA_LOGGER", "%s", out.str().c_str());
#endif
    if (m_fatal)
        std::exit(-1);
}

m3cLoggerStream &m3cLoggerStream::operator <<(const std::string &v)
{
    out << v << " ";
    return *this;
}

m3cLoggerStream &m3cLoggerStream::operator <<(const int v)
{
    out << v << " ";
    return *this;
}

m3cLoggerStream &m3cLoggerStream::operator <<(const double v)
{
    out.precision(4);
    out << v << " ";
    return *this;
}

m3cLoggerStream m3cLogger::fatal()
{
    return m3cLoggerStream(std::string("Error: "), true);
}

m3cLoggerStream m3cLogger::warning()
{
    return m3cLoggerStream(std::string("Warning: "), false);
}

m3cLoggerStream m3cLogger::info()
{
    return m3cLoggerStream(std::string("Info: "), false);
}

}
