#include "m3iRoot.h"
#include "m3iModel.h"
#include "m3iLight.h"
#include "m3iScene.h"
#include "m3iSystem.h"
#include "m3cLogger.h"
#include "m3iCamera.h"
#include "m3sAnimation.h"
#include "m3iParticles.h"
#include "m3iPhysicsBody.h"
#include "m3iBatchContext.h"
#include "m3sBatchDrawData.h"
#include "m3iRootApplication.h"
#include "m3iBatchDrawDelegate.h"
#include "m3iParticlesBehaviour.h"

using namespace melissa;

#define SHADER_SRC(v) #v

class DebugOutput
{
public:
    DebugOutput(m3iSystem *system)
        : m_system(system),
          m_time(m_system->getTime())
    {
        //m3cLogger::warning() << "created debug output";
    }

    bool update()
    {
        double t = m_system->getTime();
        double dt = t - m_time;
        m_time = t;
        m_timer += dt;
        ++m_fps;
        if (m_timer >= 1)
        {
            //m3cLogger::warning() << "Fps:" << m_fps;
            m_timer = 0;
            m_fps = 0;
            return true;
        }
        return false;
    }

private:
    m3iSystem *m_system = 0;
    double m_time = 0;
    double m_timer = 0;
    int m_fps = 0;
};

std::string glow = SHADER_SRC(
            int samples = 3;\n
        vec2 size = SCREEN_SIZE;\n
        float quality = 4.5;\n
        vec4 source = texture2D(TEXTURE, texcoord);\n
        vec4 sum = vec4(0.0);\n
        int diff = (samples - 1) / 2;\n
        vec2 sizeFactor = vec2(1) / size * quality;\n
        for (int x = -diff; x <= diff; x++)\n
{\n
            for (int y = -diff; y <= diff; y++)\n
    {\n
                vec2 offset = vec2(x, y) * sizeFactor;\n
                sum += texture2D(TEXTURE, texcoord + offset);\n
    }\n
}\n
        gl_FragColor = ((sum / float(samples * samples)) + source);\n
        );

class NewFmtTest : public m3iRootApplication
{
public:
    void initialize(m3iSystem *system, m3iRoot *root)
    {
        //getRoot()->getScene()->setLightingEnabled(false);
        getRoot()->getScene()->setSunDirection(glm::vec3(0.5, -0.7, 0.3));
        getRoot()->getScene()->setSunColor(glm::vec4(0.1, 0.2, 0.3, 1.0));

        m3iScene *s = getRoot()->getScene();
        s->setShadowMapScale(3);

        getRoot()->getScene()->getCamera()->setPosition(glm::vec3(0, 1, -3));

        std::vector<m3sModelSet> modelSet = getRoot()->getScene()->loadModelSets("demoscene.m3json");
        for (const m3sModelSet &set : modelSet)
        {
            if (set.name.substr(0, 3) != "cra")
                s->attachPhysicsMeshBody(set.model, 0);
            else
                s->attachPhysicsBoxBody(set.model, 1, glm::vec3(0.3));
        }

        m3iModel *lamp = getRoot()->getScene()->loadModelSets("lamp.m3json").at(0).model;
        lamp->setPosition(glm::vec3(0, 0, 0));
        lamp->setEuler(glm::vec3(0, 0, 0));
        lamp->setScale(glm::vec3(0.1));

        m_light = getRoot()->getScene()->createLight();

        lamp->setParent(m_light);

        m_movable = getRoot()->getScene()->createMovable();
        m_light->setParent(m_movable);
        m_light->setPosition(glm::vec3(0, 2, 0));
        m_light->setAttenuation(glm::vec3(0.1, 0.1, 0.1));
    }

    std::string loadingScreenFilename() const
    {
        return "loading.png";
    }

    void render(double dt)
    {
    }

    void touch(m3eInputAction act, uint8_t id, uint16_t x, uint16_t y)
    {
        if (act == m3eInputActionDown)
        {
            m_x = x;
            m_y = y;
        }
        if (act == m3eInputActionMove)
        {
            float sx = float (x) - m_x;
            float sy = float (y) - m_y;
            m_movable->pitch(-sy * 0.2, false);
            m_movable->yaw(sx * 0.2, true);
            m_x = x;
            m_y = y;
        }
    }

private:
    float m_x = 0;
    float m_y = 0;
    m3iMovable *m_movable = 0;
    m3iLight *m_light = 0;
};

class NewFmtTest2 : public m3iRootApplication
{
public:
    void initialize(m3iSystem *system, m3iRoot *root)
    {
        getRoot()->getScene()->setShadowsEnabled(false);
        //getRoot()->getScene()->setLightingEnabled(false);

        m3iScene *s = getRoot()->getScene();
        s->setSunColor(glm::vec4(0, 0, 0, 1));

        m3iLight *l = s->createLight();
        l->setPosition(glm::vec3(0, 5, 0));
        l->setDiffuseColor(glm::vec4(0.4, 0.5, 0.6, 1.0));

        getRoot()->getScene()->getCamera()->setPosition(glm::vec3(-3, 7, -3));
        getRoot()->getScene()->getCamera()->setEuler(glm::vec3(50, 40, 0));

        std::vector<m3sModelSet> modelSet = getRoot()->getScene()->loadModelSets("lm.m3json");

        getRoot()->getScene()->attachPostEffect(glow, 0, 0);
    }

    std::string loadingScreenFilename() const
    {
        return "loading.png";
    }

    void render(double dt)
    {
        coef += dt;
        m3iCamera *c = getRoot()->getScene()->getCamera();
        c->setEuler(glm::vec3(50 + sin(coef) * 10, 40 + cos(coef) * 10, 0));
    }

private:
    float coef = 0;
};

m3iRootApplication *createRootApplication()
{
    return new NewFmtTest2;
}
