#include "m3cPhysicsBody.h"
#include "m3iPhysicsWorldManager.h"

namespace melissa
{

m3cPhysicsBody::m3cPhysicsBody(m3iPhysicsWorldManager *worldMan, btCollisionShape *shape, float mass)
    : m_worldMan(worldMan),
      m_shape(shape)
{
    m_state = new btDefaultMotionState;

    btVector3 inertia(0, 0, 0);
    if (mass != 0.0f)
        shape->calculateLocalInertia(mass, inertia);

    btRigidBody::btRigidBodyConstructionInfo rigidBodyCI(mass, m_state, m_shape, inertia);
    m_body = new btRigidBody(rigidBodyCI);

    m_worldMan->registerPhysicsBody(this);
}

void m3cPhysicsBody::setModelMatrix(const glm::mat4 &matrix)
{
    btTransform tr;
    tr.setFromOpenGLMatrix(&matrix[0][0]);
    m_body->setWorldTransform(tr);
}

glm::mat4 m3cPhysicsBody::getModelMatrix() const
{
    glm::mat4 result;
    m_body->getWorldTransform().getOpenGLMatrix(&result[0][0]);
    return result;
}

btRigidBody *m3cPhysicsBody::getRigidBody() const
{
    return m_body;
}

m3cPhysicsBody::~m3cPhysicsBody()
{
    delete m_state;
    delete m_shape;
    m_worldMan->unregisterPhysicsBody(this);
}

}
