#pragma once

#include <btBulletDynamicsCommon.h>
#include <btBulletCollisionCommon.h>

#include "m3hGlm.h"
#include "m3iPhysicsWorldManager.h"

namespace melissa
{

class m3iMesh;
class m3iPhysicsBody;
class m3iPhysicsWorldListener;

class m3cPhysicsWorld : public m3iPhysicsWorldManager
{
public:
    m3cPhysicsWorld(m3iPhysicsWorldListener *listener);
    ~m3cPhysicsWorld();

    m3iPhysicsBody *createBoxBody(float mass, const glm::vec3 &dims);
    m3iPhysicsBody *createSphereBody(float mass, float radius);
    m3iPhysicsBody *createMeshBody(float mass, const m3iMesh *mesh);
    void simulate(float dt);

    void registerPhysicsBody(m3iPhysicsBody *body);
    void unregisterPhysicsBody(m3iPhysicsBody *body);

private:
    bool m_dontUnregisterBodies = false;
    std::vector<m3iPhysicsBody*> m_registeredBodies;

    btBroadphaseInterface *m_broadphase;
    btDefaultCollisionConfiguration *m_collisionConfiguration;
    btCollisionDispatcher *m_dispatcher;
    btSequentialImpulseConstraintSolver *m_solver;
    btDiscreteDynamicsWorld *m_world;
    m3iPhysicsWorldListener *m_listener;

    // TODO:
    std::vector<btTriangleMesh*> m_cachedTriMeshes;
};

}
