#include "m3iApplication.h"
#include "m3iRootApplication.h"
#include "m3cAndroidSystem.h"
#include "m3cAndroidFileSystem.h"
#include "m3iSound.h"

#include <sys/time.h>
#include <stdio.h>
#include <unistd.h>

namespace melissa
{

AAssetManager *g_assetManager = 0x0;
m3iApplication *g_application = 0;
m3iSystem *g_system = 0;

m3cAndroidSystem::m3cAndroidSystem()
    : m_fileSystem(new m3cAndroidFileSystem)
{
}

m3cAndroidSystem::~m3cAndroidSystem()
{
    delete m_fileSystem;
}

m3iFileSystem *m3cAndroidSystem::getFileSystem() const
{
    return m_fileSystem;
}

double m3cAndroidSystem::getTime() const
{
    struct timeval  tv;
    gettimeofday(&tv, NULL);
    return (double(tv.tv_sec) * 1000.0 + double(tv.tv_usec) / 1000.0) * 0.001;
}

}

extern "C" {

bool g_isActive = true;
JNIEnv * g_env = 0x0;

JNIEXPORT void Java_org_melissa_MelissaLib_onInit(JNIEnv * env, jobject obj, jint width, jint height, jobject assetManager);
JNIEXPORT void Java_org_melissa_MelissaLib_onFrame(JNIEnv * env, jobject obj);
JNIEXPORT void Java_org_melissa_MelissaLib_onTouch(JNIEnv * env, jobject obj, jint x, jint y, jint id, jint action);
JNIEXPORT void Java_org_melissa_MelissaLib_onKey(JNIEnv * env, jobject obj, jint key, jint action);
JNIEXPORT void Java_org_melissa_MelissaLib_onPause(JNIEnv * env, jobject obj);

JNIEXPORT void Java_org_melissa_MelissaLib_onInit(JNIEnv * env, jobject obj, jint width, jint height, jobject assetManager)
{
    g_env = env;
    melissa::g_assetManager = AAssetManager_fromJava(env, assetManager);
    static bool inited = false;
    if(!inited)
    {
        melissa::g_system = new melissa::m3cAndroidSystem;
        melissa::g_application = createApplication(createRootApplication());
        melissa::g_application->initialize(melissa::g_system);
        inited = true;
    }
    else
    {
        g_env = env;
        melissa::g_application->reload();
        if(g_isActive)
            return;
// TODO: invoke resume
        g_isActive = true;
    }
}

JNIEXPORT void Java_org_melissa_MelissaLib_onFrame(JNIEnv * env, jobject obj)
{
    g_env = env;
    melissa::g_application->render();
}

JNIEXPORT void Java_org_melissa_MelissaLib_onTouch(JNIEnv * env, jobject obj, jint x, jint y, jint id, jint action)
{
    g_env = env;

    melissa::m3eInputAction a;
    if (action == 1)
        a = melissa::m3eInputActionDown;
    if (action == 2)
        a = melissa::m3eInputActionUp;
    if (action == 3)
        a = melissa::m3eInputActionMove;

    melissa::g_application->touch(a, id, x, y);
}

JNIEXPORT void Java_org_melissa_MelissaLib_onKey(JNIEnv * env, jobject obj, jint key, jint action)
{
    g_env = env;
    melissa::g_application->keyboard(action == 1 ? melissa::m3eInputActionDown : melissa::m3eInputActionUp, key);
}

JNIEXPORT void Java_org_melissa_MelissaLib_onPause(JNIEnv * env, jobject obj)
{
    g_env = env;
    if(!g_isActive)
        return;
    melissa::g_application->stop();
    g_isActive = false;
}
/*
void playMusic(const char *fileName)
{
    jstring jstr = g_env->NewStringUTF(fileName);
    jclass clazz = g_env->FindClass("com/malcdevelop/tinyengine/MteSoundSystem");
    jmethodID methodId = g_env->GetStaticMethodID(clazz, "playMusic", "(Ljava/lang/String;)V");
    g_env->CallStaticVoidMethod(clazz, methodId, jstr);
}

void pauseMusic()
{
    jclass clazz = g_env->FindClass("com/malcdevelop/tinyengine/MteSoundSystem");
    jmethodID methodId = g_env->GetStaticMethodID(clazz, "pauseMusic", "()V");
    g_env->CallStaticVoidMethod(clazz, methodId);
}

void resumeMusic()
{
    jclass clazz = g_env->FindClass("com/malcdevelop/tinyengine/MteSoundSystem");
    jmethodID methodId = g_env->GetStaticMethodID(clazz, "resumeMusic", "()V");
    g_env->CallStaticVoidMethod(clazz, methodId);
}

void stopMusic()
{
    jclass clazz = g_env->FindClass("com/malcdevelop/tinyengine/MteSoundSystem");
    jmethodID methodId = g_env->GetStaticMethodID(clazz, "stopMusic", "()V");
    g_env->CallStaticVoidMethod(clazz, methodId);
}

int openSound(const char *fileName)
{
    jstring jstr = g_env->NewStringUTF(fileName);
    jclass clazz = g_env->FindClass("com/malcdevelop/tinyengine/MteSoundSystem");
    jmethodID methodId = g_env->GetStaticMethodID(clazz, "openSound", "(Ljava/lang/String;)I");
    return g_env->CallStaticIntMethod(clazz, methodId, jstr);
}

void playSound(int snd, int repeats)
{
    jclass clazz = g_env->FindClass("com/malcdevelop/tinyengine/MteSoundSystem");
    jmethodID methodId = g_env->GetStaticMethodID(clazz, "playSound", "(II)V");
    g_env->CallStaticVoidMethod(clazz, methodId, snd, repeats);
}*/

}

namespace melissa
{

class m3cSoundAndroid : public m3iSound
{
public:
    m3cSoundAndroid(const std::string &filename)
    {
        jstring jstr = g_env->NewStringUTF(filename.c_str());
        jclass clazz = g_env->FindClass("org/melissa/MelissaSound");
        jmethodID methodId = g_env->GetStaticMethodID(clazz, "openSound", "(Ljava/lang/String;)I");
        m_id = g_env->CallStaticIntMethod(clazz, methodId, jstr);
    }

    void play(int repeats)
    {
        jclass clazz = g_env->FindClass("org/melissa/MelissaSound");
        jmethodID methodId = g_env->GetStaticMethodID(clazz, "playSound", "(II)V");
        g_env->CallStaticVoidMethod(clazz, methodId, m_id, repeats);
    }

private:
    int m_id = -1;
};

void m3cAndroidSystem::playMusic(const std::string &filename)
{
    jstring jstr = g_env->NewStringUTF(filename.c_str());
    jclass clazz = g_env->FindClass("org/melissa/MelissaSound");
    jmethodID methodId = g_env->GetStaticMethodID(clazz, "playMusic", "(Ljava/lang/String;)V");
    g_env->CallStaticVoidMethod(clazz, methodId, jstr);
}

void m3cAndroidSystem::stopMusic()
{
    jclass clazz = g_env->FindClass("org/melissa/MelissaSound");
    jmethodID methodId = g_env->GetStaticMethodID(clazz, "stopMusic", "()V");
    g_env->CallStaticVoidMethod(clazz, methodId);
}

void m3cAndroidSystem::pauseMusic()
{
    jclass clazz = g_env->FindClass("org/melissa/MelissaSound");
    jmethodID methodId = g_env->GetStaticMethodID(clazz, "pauseMusic", "()V");
    g_env->CallStaticVoidMethod(clazz, methodId);
}

void m3cAndroidSystem::resumeMusic()
{
    jclass clazz = g_env->FindClass("org/melissa/MelissaSound");
    jmethodID methodId = g_env->GetStaticMethodID(clazz, "resumeMusic", "()V");
    g_env->CallStaticVoidMethod(clazz, methodId);
}

m3iSound *m3cAndroidSystem::getSound(const std::string &filename)
{
    return new m3cSoundAndroid(filename);
}

void m3cAndroidSystem::saveParameter(const std::string &name, const std::string &value)
{
    jstring jstr = g_env->NewStringUTF(name.c_str());
    jstring jstr1 = g_env->NewStringUTF(value.c_str());
    jclass clazz = g_env->FindClass("org/melissa/MelissaActivity");
    jmethodID methodId = g_env->GetStaticMethodID(clazz, "saveParameter", "(Ljava/lang/String;Ljava/lang/String;)V");
    g_env->CallStaticVoidMethod(clazz, methodId, jstr, jstr1);
}

std::string m3cAndroidSystem::loadParameter(const std::string &name)
{
    jstring jstr = g_env->NewStringUTF(name.c_str());
    jclass clazz = g_env->FindClass("org/melissa/MelissaActivity");
    jmethodID methodId = g_env->GetStaticMethodID(clazz, "loadParameter", "(Ljava/lang/String;)Ljava/lang/String;");

    jobject result = g_env->CallStaticObjectMethod(clazz, methodId, jstr);
    const char* str = g_env->GetStringUTFChars((jstring) result, NULL);

    return str;
}

}
