#include "m3iApplication.h"
#include "m3cDesktopSystem.h"
#include "m3cDiskFileSystem.h"
#include "m3cSoundDesktop.h"
#include "m3cLogger.h"
#include "m3hOpenGl.h"

#if defined(_WIN32) || defined(_WIN64)
    #include <windows.h>
#endif

#define FREEGLUT_STATIC
#include <GL/freeglut.h>

#include <sys/time.h>
#include <stdio.h>
#include <unistd.h>

namespace melissa
{

m3cDesktopSystem::m3cDesktopSystem()
    : m_fileSystem(new m3cDiskFileSystem)
{
}

m3cDesktopSystem::~m3cDesktopSystem()
{
    delete m_fileSystem;
}

m3iFileSystem *m3cDesktopSystem::getFileSystem() const
{
    return m_fileSystem;
}

double m3cDesktopSystem::getTime() const
{
    struct timeval  tv;
    gettimeofday(&tv, NULL);
    return (double(tv.tv_sec) * 1000.0 + double(tv.tv_usec) / 1000.0) * 0.001;
}

void m3cDesktopSystem::playMusic(const std::string &filename)
{
}

void m3cDesktopSystem::stopMusic()
{
}

void m3cDesktopSystem::pauseMusic()
{
}

void m3cDesktopSystem::resumeMusic()
{
}

m3iSound *m3cDesktopSystem::getSound(const std::string &filename)
{
    return new m3cSoundDesktop;
}

void m3cDesktopSystem::saveParameter(const std::string &name, const std::string &value)
{
    // TODO:
}

std::string m3cDesktopSystem::loadParameter(const std::string &name)
{
    return "";
}

}

using namespace melissa;

m3cDesktopSystem *g_system = 0;
m3iApplication *g_application = 0;

void glut_init()
{
    g_system = new m3cDesktopSystem;
    g_application->initialize(g_system);
}

void glut_reshape(int w, int h)
{
    glViewport(0, 0, w, h);
    g_application->reload();
}

void glut_keyboard(unsigned char key, int x, int y)
{
    g_application->keyboard(m3eInputActionDown, key);
}

void glut_keyboard_up(unsigned char key, int x, int y)
{
    g_application->keyboard(m3eInputActionUp, key);
}

void glut_mouse(int button, int state, int x, int y)
{
    g_application->touch(state == GLUT_DOWN ? m3eInputActionDown : m3eInputActionUp, button, x, y);
}

void glut_motion(int x, int y)
{
    g_application->touch(m3eInputActionMove, 0, x, y);
}

void glut_render()
{
    g_application->render();
    glutSwapBuffers();
}

void glut_close()
{
    g_application->close();
    delete g_application;
}

int MELISSA_API glut_main(melissa::m3iApplication *app, int argc, char** argv);

int glut_main(melissa::m3iApplication *app, int argc, char** argv)
{
    g_application = app;

    auto o = g_application->getSimulatorOrientation();

    uint16_t width = o == m3eScreenOrientationLandscape ? 800 : 480;
    uint16_t height = o == m3eScreenOrientationLandscape ? 480 : 800;

    glutInit(&argc, argv);
    glutInitWindowSize(width, height);
    glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
    glutCreateWindow("Simulator");

    if (glewInit() != GLEW_OK)
    {
        melissa::m3cLogger::fatal() << "Can't init GLEW";
        exit(-255);
    }

    m3cLogger::info() << "GLEW inited";
    glut_init();

    glutReshapeFunc(glut_reshape);
    glutKeyboardFunc(glut_keyboard);
    glutKeyboardUpFunc(glut_keyboard_up);
    glutMouseFunc(glut_mouse);
    glutMotionFunc(glut_motion);
    glutIdleFunc(glut_render);
    glutCloseFunc(glut_close);

    glutMainLoop ();
  return 0;
}
