#include "m3iSkeleton.h"
#include "m3cAbstractMesh.h"
#include "m3iResourceManager.h"

namespace melissa
{

m3cAbstractMesh::m3cAbstractMesh(m3iResourceManager *resMan, const m3sMeshData &data)
    : m_data(data),
      m_resMan(resMan)
{
    m_resMan->registerResource(this);
    calculateAABB();
}

m3cAbstractMesh::~m3cAbstractMesh()
{
    m_resMan->unregisterResource(this);
    delete m_data.skeleton;
}

std::vector<m3sMeshSurface> &m3cAbstractMesh::editSurfaces()
{
    return m_data.surfaces;
}

void m3cAbstractMesh::calculateAABB()
{
    for (uint64_t i = 0; i < m_data.verticesCount; ++i)
    {
        const m3sVertex &v = m_data.vertices[i];
        m_aabb[0].x = glm::min(v.position.x, m_aabb[0].x);
        m_aabb[0].y = glm::min(v.position.y, m_aabb[0].y);
        m_aabb[0].z = glm::min(v.position.z, m_aabb[0].z);

        m_aabb[1].x = glm::max(v.position.x, m_aabb[1].x);
        m_aabb[1].y = glm::max(v.position.y, m_aabb[1].y);
        m_aabb[1].z = glm::max(v.position.z, m_aabb[1].z);
    }

    float a = glm::abs(m_aabb[0].x  - m_aabb[1].x);
    float b = glm::abs(m_aabb[0].y  - m_aabb[1].y);
    float c = glm::abs(m_aabb[0].z  - m_aabb[1].z);
    m_boundingCubeSize = glm::max(a, glm::max(b, c));
}

const m3sMeshData &m3cAbstractMesh::getData() const
{
    return m_data;
}

float m3cAbstractMesh::getBoundingCubeSize() const
{
    return m_boundingCubeSize;
}

const m3sSurfaceMaterial &m3cAbstractMesh::getSurfaceMaterial(uint64_t index) const
{
    return m_data.surfaces[index].material;
}

void m3cAbstractMesh::setSurfaceMaterial(uint64_t index, const m3sSurfaceMaterial &material)
{
    m_data.surfaces[index].material = material;
}

}
