#pragma once

#include "m3hStd.h"
#include "m3hOpenGl.h"
#include "m3iResource.h"

namespace melissa
{

class m3iResourceManager;

struct m3sProgramHandles
{
    GLint AttributePosition = -1;
    GLint AttributeNormal = -1;
    GLint AttributePositionNext = -1;
    GLint AttributeNormalNext = -1;
    GLint AttributeTexcoord1 = -1;
    GLint AttributeTexcoord2 = -1;
    GLint AttributeColor = -1;
    GLint AttributeJoint = -1;
    GLint AttributeWeight = -1;
    GLint AttributeTangent = -1;
    GLint AttributeBitangent = -1;

    GLint UniformM = -1;
    GLint UniformMv = -1;
    GLint UniformMvp = -1;
    GLint UniformShadowMapBiasMatrix = -1;
    GLint UniformNormalMatrix = -1;
    GLint UniformTexture = -1;
    GLint UniformNormalMapTexture = -1;
    GLint UniformLightMapTexture = -1;
    GLint UniformShadowMapTexture = -1;
    GLint UniformScreenTexture = -1;
    GLint UniformMaterialDiffuse = -1;
    GLint UniformMaterialSpecular = -1;
    GLint UniformScreenSize = -1;
    GLint UniformFrameTime = -1;
    GLint UniformJointMatrix = -1;
    GLint UniformFogColor = -1;
    GLint UniformFogStart = -1;
    GLint UniformFogEnd = -1;

    GLint UniformCameraPosition = -1;

    GLint UniformSunColor = -1;
    GLint UniformSunPosition = -1;
    GLint UniformSunDirection = -1;

    GLint UniformLightsCount = -1;
    GLint UniformLightDiffuse[4] = {-1, -1, -1, -1};
    GLint UniformLightPosition[4] = {-1, -1, -1, -1};
    GLint UniformLightDirection[4] = {-1, -1, -1, -1};
    GLint UniformLightAttenuation[4] = {-1, -1, -1, -1};
};

class m3cProgram : public m3iResource
{
public:
    m3cProgram(m3iResourceManager *resMan, const std::string &source);
    ~m3cProgram();

    const m3sProgramHandles &getProgramHandles() const;
    void enableAttributes();
    void disableAttributes();
    void use();

    void freeHandles();
    void reloadHandles();

    static const uint8_t getMaxLights();
    static const uint8_t getMaxJoints();
    static void resetCurrent();

private:
    void initializeHandles();
    GLuint createShaderFromSource(const std::string &source, GLenum type) const;

private:
    static m3cProgram *s_program;

    GLuint m_program = 0;
    std::string m_source;
    m3sProgramHandles m_handles;
    m3iResourceManager *m_resMan;
};

}
