#include "m3cLogger.h"
#include "m3cTexture.h"
#include "m3iResourceManager.h"

namespace melissa
{

m3cTexture::m3cTexture(m3iResourceManager *resMan, uint16_t width, uint16_t height, const void *data, uint64_t size, m3cTexture::Format format)
    : m_width(width),
      m_height(height),
      m_resMan(resMan),
      m_format(format)
{
    if (size)
        m_data = std::vector<uint8_t>(reinterpret_cast<const uint8_t*>(data), reinterpret_cast<const uint8_t*>(data) + size);
    m_resMan->registerResource(this);
    reloadHandles();
}

m3cTexture::~m3cTexture()
{
    freeHandles();
    m_resMan->unregisterResource(this);
}

void m3cTexture::bind(uint8_t layer)
{
    glActiveTexture(GL_TEXTURE0 + layer);
    glBindTexture(GL_TEXTURE_2D, m_texture);
}

GLuint m3cTexture::getHandle() const
{
    return m_texture;
}

uint16_t m3cTexture::getWidth() const
{
    if (m_width == 0)
        return m_resMan->getViewport().z;
    return m_width;
}

uint16_t m3cTexture::getHeight() const
{
    if (m_height == 0)
        return m_resMan->getViewport().w;
    return m_height;
}

void m3cTexture::freeHandles()
{
    glDeleteTextures(1, &m_texture);
}

void m3cTexture::reloadHandles()
{
    glGenTextures(1, &m_texture);
    bind(0);

    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);

    GLenum fmt = GL_UNSIGNED_BYTE;
    if (m_format == Float)
        fmt = GL_FLOAT;

    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, getWidth(), getHeight(), 0, GL_RGBA, fmt, m_data.size() ? m_data.data() : 0);
}

void m3cTexture::updateData(const void *data, uint64_t size)
{
    if (size)
        m_data = std::vector<uint8_t>(reinterpret_cast<const uint8_t*>(data), reinterpret_cast<const uint8_t*>(data) + size);
    else
        m_data.clear();

    GLenum fmt = GL_UNSIGNED_BYTE;
    if (m_format == Float)
        fmt = GL_FLOAT;

    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, getWidth(), getHeight(), 0, GL_RGBA, fmt, m_data.size() ? m_data.data() : 0);
}

}
