#include "m3cLogger.h"
#include "m3cTexture.h"
#include "m3iResourceManager.h"
#include "m3cTextureRenderTarget.h"

namespace melissa
{

m3cTextureRenderTarget::m3cTextureRenderTarget(m3iResourceManager *resMan, m3cTexture *texture)
    : m_texture(texture),
      m_resMan(resMan)
{
    m_resMan->registerResource(this);
    reloadHandles();
}

m3cTextureRenderTarget::~m3cTextureRenderTarget()
{
    freeHandles();
    m_resMan->unregisterResource(this);
}

m3cTexture *m3cTextureRenderTarget::getTexture() const
{
    return m_texture;
}

void m3cTextureRenderTarget::bind()
{
    glBindFramebuffer(GL_FRAMEBUFFER, m_fb);
    glBindRenderbuffer(GL_RENDERBUFFER, m_rb);

    if (m_reloadMode)
    {
        glRenderbufferStorage(GL_RENDERBUFFER, GL_DEPTH_COMPONENT16, m_texture->getWidth(), m_texture->getHeight());

        glFramebufferRenderbuffer(GL_FRAMEBUFFER,
                                  GL_DEPTH_ATTACHMENT,
                                  GL_RENDERBUFFER,
                                  m_rb);

        glFramebufferTexture2D(GL_FRAMEBUFFER,
                               GL_COLOR_ATTACHMENT0,
                               GL_TEXTURE_2D,
                               m_texture->getHandle(),
                               0);

        if(glCheckFramebufferStatus(GL_FRAMEBUFFER) != GL_FRAMEBUFFER_COMPLETE)
        {
            m3cLogger::fatal() << "Can't create framebuffer";
        }
    }

    m_reloadMode = false;
}

void m3cTextureRenderTarget::unbind()
{
    glBindFramebuffer(GL_FRAMEBUFFER, 0);
    glBindRenderbuffer(GL_RENDERBUFFER, 0);
}

void m3cTextureRenderTarget::freeHandles()
{
    glDeleteFramebuffers(1, &m_fb);
    glDeleteRenderbuffers(1, &m_rb);
}

void m3cTextureRenderTarget::reloadHandles()
{
    m_reloadMode = true;

    glGenFramebuffers(1, &m_fb);
    glGenRenderbuffers(1, &m_rb);
}

}
