#pragma once

#include "m3hStd.h"
#include "m3hGlm.h"

namespace melissa
{

class m3iMesh;
class m3iTexture;

struct m3sMeshData;
struct m3sDrawMeshData;
struct m3sRenderFrameData;

class m3iRender
{
public:
    virtual ~m3iRender() {}
    virtual m3iMesh *createHardwareMesh(const m3sMeshData &data, bool optimize) = 0;
    virtual m3iMesh *createLinkedMesh(const m3sMeshData &data) = 0;
    virtual m3iTexture *createTexture(uint16_t width, uint16_t height, const void *data, uint64_t size) = 0;

    virtual uint64_t createPostEffectSet() = 0;
    virtual void clearPostEffectSet(uint64_t id) = 0;
    virtual uint64_t getCurrentPostEffectSet() const = 0;
    virtual void usePostEffectSet(uint64_t id) = 0;

    virtual uint64_t attachPostEffect(const std::string &fragmentSource, uint16_t width, uint16_t height) = 0;
    virtual void detachPostEffect(uint64_t id) = 0;

    virtual void setFrameData(const m3sRenderFrameData &data) = 0;
    virtual void clear() = 0;
    virtual void drawMesh(const m3sDrawMeshData &data) = 0;
    virtual void flush() = 0;

    virtual void reload() = 0;

    virtual glm::vec4 getViewport() const = 0;
};

}
