#include "m3iMesh.h"
#include "m3cModel.h"
#include "m3iRender.h"
#include "m3sMeshData.h"
#include "m3sMeshSurface.h"
#include "m3iMovableManager.h"

namespace melissa
{

m3cModel::m3cModel(m3iMovableManager *movMan, m3iMesh *mesh)
    : m3cMovable(movMan)
{
    m_data.mesh = mesh;
}

void m3cModel::draw(m3iRender *render, m3sRenderFrameData &frameData)
{
    if (!isVisible())
        return;
    m_data.modelMatrix = getModelMatrix(true);
    m_data.frameTime = m_frameTime;
    render->drawMesh(m_data);
}

void m3cModel::update(double dt)
{
    m_frameTime += dt * double(m_anim.fps);
    if (m_frameTime < m_anim.start)
        m_frameTime = m_anim.start;
    if (m_frameTime >= m_anim.stop)
    {
        if (m_anim.loop)
            m_frameTime = m_anim.start;
        else
            m_frameTime = m_anim.stop;
    }
}

float m3cModel::getBoundingCubeSize() const
{
    return m_data.mesh->getBoundingCubeSize();
}

void m3cModel::setCastShadowsEnabled(bool enabled)
{
    m_data.castShadows = enabled;
}

void m3cModel::setReceiveShadowsEnabled(bool enabled)
{
    m_data.receiveShadows = enabled;
}

m3iMesh *m3cModel::getMesh() const
{
    return m_data.mesh;
}

uint64_t m3cModel::getMeshSurfacesCount() const
{
    return getMesh()->getData().surfaces.size();
}

const m3sSurfaceMaterial &m3cModel::getMeshSurfaceMaterial(uint64_t index) const
{
    return getMesh()->getSurfaceMaterial(index);
}

void m3cModel::setMeshSurfaceMaterial(uint64_t index, const m3sSurfaceMaterial &material)
{
    getMesh()->setSurfaceMaterial(index, material);
}

void m3cModel::setAnimation(const m3sAnimation &anim)
{
    m_anim = anim;
}

void m3cModel::setBlendingEnabled(bool enabled)
{
    m_data.blending = true;
}

bool m3cModel::isBlendingEnabled() const
{
    return m_data.blending;
}

}
