#pragma once

#include "m3iMovable.h"

namespace melissa
{

class m3iMovableManager;

class m3cMovable : public virtual m3iMovable
{
public:
    m3cMovable(m3iMovableManager *movMan);
    ~m3cMovable();

    void addPosition(const glm::vec3 &position);
    void setPosition(const glm::vec3 &position);
    void setEuler(const glm::vec3 &rotation);
    void setQuaternion(const glm::quat &quaternion);
    void setScale(const glm::vec3 &scale);
    void pitch(float v, bool global);
    void yaw(float v, bool global);
    void roll(float v, bool global);
    void rotate(const glm::quat &orientation, bool global);
    void addScale(const glm::vec3 &scale);
    glm::vec3 getPosition(bool global) const;
    glm::vec3 getDirection(bool global) const;
    glm::vec3 getEuler(bool global) const;
    glm::vec3 getScale(bool global) const;
    glm::mat4 getModelMatrix(bool global) const;
    void setModelMatrix(const glm::mat4 &matrix);
    void setParent(const m3iMovable *parent);
    void lookTo(const glm::vec3 &position);

protected:
    glm::vec3 getScaleFromMatrix(const glm::mat4 &mat) const;

protected:
    mutable bool m_modify = true;

    glm::vec3 m_translation = {0, 0, 0};
    glm::quat m_orientation = glm::quat();
    glm::vec3 m_scale = {1, 1, 1};

    mutable glm::mat4 m_mat = glm::mat4(1.0);
    const m3iMovable *m_parent = 0;
    m3iMovableManager *m_movMan;
};

}
