#pragma once

#include "m3hStd.h"
#include "m3iScene.h"
#include "m3iTextureManager.h"
#include "m3iMovableManager.h"
#include "m3cModelSetsLoader.h"
#include "m3sRenderFrameData.h"
#include "m3iPhysicsWorldListener.h"

namespace melissa
{

class m3iMesh;
class m3cBatch;
class m3cModel;
class m3cLight;
class m3iRender;
class m3iSystem;
class m3cCamera;
class m3iMovable;
class m3cParticles;
class m3cPhysicsBody;
class m3cPhysicsWorld;
class m3iSceneManager;

class m3cScene : public m3iScene, public m3iMovableManager, public virtual m3iTextureManager, public m3iPhysicsWorldListener
{
public:
    m3cScene(m3iRender *render, m3iSystem *system, m3iSceneManager *sceneMan);
    ~m3cScene();

    void reload();
    m3iBatch *createBatch(const glm::vec2 &spaceSize, const std::string &filename, m3iBatchDrawDelegate *drawDel, uint64_t max, bool is2d);
    m3iMovable *createMovable();
    std::vector<m3sModelSet> loadModelSets(const std::string &filename);
    m3iModel *createModel(m3iMesh *mesh);
    m3iCamera *createCamera(bool setAsCurrent);
    void setCamera(m3iCamera *camera);
    m3iCamera *getCamera() const;
    m3iLight *createLight();
    m3iParticles *createParticles(m3iParticlesBehaviour *behav, const std::string &filename, uint64_t count);
    void render(double dt);
    m3iTexture *loadTexture(const std::string &filename);
    void setShadowMapScale(float scale);
    void setShadowsEnabled(bool enabled);
    void setLightingEnabled(bool enabled);

    void registerMovable(m3iMovable *movable);
    void unregisterMovable(m3iMovable *movable);

    uint64_t attachPostEffect(const std::string &fragmentSource, uint16_t width, uint16_t height);
    void detachPostEffect(uint64_t id);

    m3iPhysicsBody *attachPhysicsBoxBody(m3iMovable *movable, float mass, const glm::vec3 &dims);
    m3iPhysicsBody *attachPhysicsSphereBody(m3iMovable *movable, float mass, float radius);
    m3iPhysicsBody *attachPhysicsMeshBody(m3iModel *model, float mass);

    void setSunColor(const glm::vec4 &color);
    void setSunPosition(const glm::vec3 &position);
    void setSunDirection(const glm::vec3 &direction);

    void setFogParams(float near, float far, const glm::vec4 &color);
    void setFogEnabled(bool enabled);
    bool isFogEnabled() const;

    void setClearColor(const glm::vec4 &color);

    void addedPhysicsBody(m3iPhysicsBody *body);
    void removedPhysicsBody(m3iPhysicsBody *body);

private:
    int m_loadingFrame = 0;
    m3cCamera *m_camera = 0;
    m3iRender *m_render = 0;
    m3iSystem *m_system = 0;
    uint64_t m_postEffectSet;
    m3iSceneManager *m_sceneMan;
    m3sRenderFrameData m_frameData;
    m3cPhysicsWorld *m_physicsWorld;
    m3cModelSetsLoader m_modelSetsLoader;
    bool m_dontUnregisterMovables = false;
    std::vector<m3cLight*> m_lightsForUse;
    std::vector<m3cModel*> m_modelsForDraw;
    std::vector<m3cParticles*> m_particlesForDraw;
    std::vector<m3cBatch*> m_batchesForDraw;
    std::vector<m3iMovable*> m_registeredMovables;
    std::map<std::string, m3iTexture*> m_cachedTextures;
    std::map<m3iMovable*, m3cPhysicsBody*> m_attachedPhysicsBodies;
};

}
