#pragma once

#include "m3iSkeleton.h"
#include "m3sSkeletonData.h"

namespace melissa
{

struct m3sSkeletonJointFrame{
    void setPosition(float x, float y, float z);
    void setRotation(float x, float y, float z, float w);

    glm::vec3 pos;
    glm::quat rot;
    int time;
};

struct m3sSkeletonJoint{
    m3sSkeletonJoint();
    // need sort frames by time for fast search
    void sortFrames();
    void setPosition(float x, float y, float z);
    void setRotation(float x, float y, float z, float w);
    void setParent(m3sSkeletonJoint *parent);
    m3sSkeletonJoint *getParent() const;
    std::vector<m3sSkeletonJoint*> &getChildren();

    // local bindPos/Rot matrix must be multiple by local pos/rot interpolated frame
    glm::vec3 bindPos;
    glm::quat bindRot;
    glm::mat4 localMat; // will calculate when build static data of skeleton (must build)
    glm::mat4 globInvMat; // will calculate when build static data of skeleton (must build)
    glm::mat4 globFrameMat; // time depends matrix
    std::vector<m3sSkeletonJointFrame> frames;

private:
    m3sSkeletonJoint *parent;
    std::vector<m3sSkeletonJoint*> children;
};

class m3cSkeleton : public m3iSkeleton
{
public:
    const std::vector<glm::mat4> &getJoints(double frameTime);
    uint64_t getJointsCount() const;

    m3cSkeleton();
    void build(const m3sSkeletonData &data);
    void addJoint(const m3sSkeletonJoint &joint);
    const m3sSkeletonJoint &getJoint(uint32_t index) const;
    m3sSkeletonJoint &getJoint(uint32_t index);
    void clear();
    void setFrame(double frameTime);
    uint32_t getCountJoints() const;
    float *getRawFinalMatrices();
    void render(m3sSkeletonJoint *joint);
    void render();

private:
    void calculateFramesMatrix(bool root, m3sSkeletonJoint &joint, const glm::mat4 &parentMatrix, double frameTime);
    glm::mat4 calculateGlobalMatrix(m3sSkeletonJoint &joint);
    bool needRebuild() const;
    void setAsRebuilt();
    void setAsNeedRebuild();

    std::vector<glm::mat4> m_finalMats;
    std::vector<m3sSkeletonJoint> m_joints;
    bool m_needRebuild;
};

}
