#include "m3iFile.h"
#include "m3iRender.h"
#include "m3cLogger.h"
#include "m3iFileSystem.h"
#include "m3cTextureLoader.h"

#include "jpgd/jpgd.h"
#include "lodepng/lodepng.h"

namespace melissa
{

m3iTexture *loadPng(m3iRender *render, m3iFile *file)
{
    std::vector<uint8_t> image;
    unsigned width, height;

    uint64_t size = file->getSize();
    std::vector<uint8_t> png(size);
    file->read(png.data(), size);

    lodepng::decode(image, width, height, png);

    std::vector<uint8_t> image_inv(image.size());
    for (int64_t j = height - 1; j >= 0; --j){
        for (int64_t i = 0; i < width; ++i){
            int j1 = height - j - 1;
            for (uint8_t k = 0; k < 4; ++k)
                image_inv[j1 * 4 * width + i * 4 + k]  = image[j * 4 * width + i * 4 + k];
        }
    }

    delete file;
    return render->createTexture(width, height, image_inv.data(), image_inv.size());
}

m3iTexture *loadJpg(m3iRender *render, m3iFile *file)
{
    using namespace jpgd;

    std::vector<uint8_t> data(file->getSize());
    file->read(data.data(), data.size());

    struct s_rgb_jpg{
        uint8_t r, g, b;
    };

    struct s_rgb_jpga{
        uint8_t r, g, b, a;
    };

    int width;
    int height;
    int comps;
    s_rgb_jpg *jpgBuffer = reinterpret_cast<s_rgb_jpg*>(decompress_jpeg_image_from_memory(data.data(), data.size(), &width, &height, &comps, 3));
    if (!jpgBuffer)
    {
        m3cLogger::fatal() << "can't load jpg file";
        return nullptr;
    }

    // vertical flip NODE: check
    std::vector<s_rgb_jpga> flipped(width * height);
    for (uint j = 0; j < height; ++j){
        for (uint32_t i = 0; i < width; ++i){
            flipped[(height - j - 1) * width + i].r = jpgBuffer[j * width + i].r;
            flipped[(height - j - 1) * width + i].g = jpgBuffer[j * width + i].g;
            flipped[(height - j - 1) * width + i].b = jpgBuffer[j * width + i].b;
            flipped[(height - j - 1) * width + i].a = 255;
        }
    }

    std::vector<uint8_t> raw((uint8_t*)&flipped[0], (uint8_t*)&flipped[flipped.size()]);

    auto result = render->createTexture(width, height, raw.data(), raw.size());
    free(jpgBuffer);

    delete file;
    return result;
}

m3iTexture *m3cTextureLoader::loadTexture(m3iRender *render, m3iFileSystem *filesystem, const std::string &filename)
{
    std::string ext = filename.substr(filename.find_last_of('.') + 1);

    if (ext == "jpg")
        return loadJpg(render, filesystem->openFile(filename));

    if (ext == "png")
        return loadPng(render, filesystem->openFile(filename));

    m3cLogger::fatal() << "can't fild loader for" << ext;
    return nullptr;
}

}
